@@ -121,7 +121,9 @@ class UCX(Comm):
             deserializers = ("cuda", "dask", "pickle", "error")
         resp = await self.ep.recv_future()
         obj = ucp.get_obj_from_msg(resp)
-        nframes, = struct.unpack("Q", obj[:8])  # first eight bytes for number of frames
+        (nframes,) = struct.unpack(
+            "Q", obj[:8]
+        )  # first eight bytes for number of frames
 
         gpu_frame_msg = obj[
             8 : 8 + nframes
