@@ -1,5 +1,3 @@
-from __future__ import print_function, division, absolute_import
-
 from functools import partial
 import logging
 import math
@@ -20,6 +18,7 @@ from bokeh.models import (
     Range1d,
     Plot,
     Quad,
+    Span,
     value,
     LinearAxis,
     NumeralTickFormatter,
@@ -132,8 +131,7 @@ class Occupancy(DashboardComponent):
                     "y": [1, 2],
                     "ms": [1, 2],
                     "color": ["red", "blue"],
-                    "dashboard_port": ["", ""],
-                    "dashboard_host": ["", ""],
+                    "escaped_worker": ["a", "b"],
                 }
             )
 
@@ -155,9 +153,7 @@ class Occupancy(DashboardComponent):
             # fig.xaxis[0].formatter = NumeralTickFormatter(format='0.0s')
             fig.x_range.start = 0
 
-            tap = TapTool(
-                callback=OpenURL(url="./proxy/@dashboard_port/@dashboard_host/status")
-            )
+            tap = TapTool(callback=OpenURL(url="./info/worker/@escaped_worker.html"))
 
             hover = HoverTool()
             hover.tooltips = "@worker : @occupancy s."
@@ -171,9 +167,6 @@ class Occupancy(DashboardComponent):
         with log_errors():
             workers = list(self.scheduler.workers.values())
 
-            dashboard_host = [ws.host for ws in workers]
-            dashboard_port = [ws.services.get("dashboard", "") for ws in workers]
-
             y = list(range(len(workers)))
             occupancy = [ws.occupancy for ws in workers]
             ms = [occ * 1000 for occ in occupancy]
@@ -202,8 +195,7 @@ class Occupancy(DashboardComponent):
                     "worker": [ws.address for ws in workers],
                     "ms": ms,
                     "color": color,
-                    "dashboard_host": dashboard_host,
-                    "dashboard_port": dashboard_port,
+                    "escaped_worker": [escape.url_escape(ws.address) for ws in workers],
                     "x": x,
                     "y": y,
                 }
@@ -318,11 +310,12 @@ class CurrentLoad(DashboardComponent):
                     "nbytes": [1, 2],
                     "nbytes-half": [0.5, 1],
                     "nbytes_text": ["1B", "2B"],
+                    "cpu": [1, 2],
+                    "cpu-half": [0.5, 1],
                     "worker": ["a", "b"],
                     "y": [1, 2],
                     "nbytes-color": ["blue", "blue"],
-                    "dashboard_port": ["", ""],
-                    "dashboard_host": ["", ""],
+                    "escaped_worker": ["a", "b"],
                 }
             )
 
@@ -363,6 +356,32 @@ class CurrentLoad(DashboardComponent):
             )
             rect.nonselection_glyph = None
 
+            cpu = figure(
+                title="CPU Utilization",
+                tools="",
+                id="bk-cpu-worker-plot",
+                width=int(width / 2),
+                name="cpu_hist",
+                **kwargs
+            )
+            rect = cpu.rect(
+                source=self.source,
+                x="cpu-half",
+                y="y",
+                width="cpu",
+                height=1,
+                color="blue",
+            )
+            rect.nonselection_glyph = None
+            hundred_span = Span(
+                location=100,
+                dimension="height",
+                line_color="gray",
+                line_dash="dashed",
+                line_width=3,
+            )
+            cpu.add_layout(hundred_span)
+
             nbytes.axis[0].ticker = BasicTicker(mantissas=[1, 256, 512], base=1024)
             nbytes.xaxis[0].formatter = NumeralTickFormatter(format="0.0 b")
             nbytes.xaxis.major_label_orientation = -math.pi / 12
@@ -374,9 +393,7 @@ class CurrentLoad(DashboardComponent):
                 fig.ygrid.visible = False
 
                 tap = TapTool(
-                    callback=OpenURL(
-                        url="./proxy/@dashboard_port/@dashboard_host/status"
-                    )
+                    callback=OpenURL(url="./info/worker/@escaped_worker.html")
                 )
                 fig.add_tools(tap)
 
@@ -394,20 +411,27 @@ class CurrentLoad(DashboardComponent):
             hover.point_policy = "follow_mouse"
             nbytes.add_tools(hover)
 
+            hover = HoverTool()
+            hover.tooltips = "@worker : @cpu %"
+            hover.point_policy = "follow_mouse"
+            cpu.add_tools(hover)
+
             self.processing_figure = processing
             self.nbytes_figure = nbytes
+            self.cpu_figure = cpu
 
             processing.y_range = nbytes.y_range
+            cpu.y_range = nbytes.y_range
 
     @without_property_validation
     def update(self):
         with log_errors():
             workers = list(self.scheduler.workers.values())
 
-            dashboard_host = [ws.host for ws in workers]
-            dashboard_port = [ws.services.get("dashboard", "") for ws in workers]
-
             y = list(range(len(workers)))
+
+            cpu = [int(ws.metrics["cpu"]) for ws in workers]
+
             nprocessing = [len(ws.processing) for ws in workers]
             processing_color = []
             for ws in workers:
@@ -442,6 +466,8 @@ class CurrentLoad(DashboardComponent):
             if any(nprocessing) or self.last + 1 < now:
                 self.last = now
                 result = {
+                    "cpu": cpu,
+                    "cpu-half": [c / 2 for c in cpu],
                     "nprocessing": nprocessing,
                     "nprocessing-half": [np / 2 for np in nprocessing],
                     "nprocessing-color": processing_color,
@@ -449,9 +475,8 @@ class CurrentLoad(DashboardComponent):
                     "nbytes-half": [nb / 2 for nb in nbytes],
                     "nbytes-color": nbytes_color,
                     "nbytes_text": nbytes_text,
-                    "dashboard_host": dashboard_host,
-                    "dashboard_port": dashboard_port,
                     "worker": [ws.address for ws in workers],
+                    "escaped_worker": [escape.url_escape(ws.address) for ws in workers],
                     "y": y,
                 }
 
@@ -1511,6 +1536,14 @@ def individual_nbytes_doc(scheduler, extra, doc):
     doc.theme = BOKEH_THEME
 
 
+def individual_cpu_doc(scheduler, extra, doc):
+    current_load = CurrentLoad(scheduler, sizing_mode="stretch_both")
+    current_load.update()
+    add_periodic_callback(doc, current_load, 100)
+    doc.add_root(current_load.cpu_figure)
+    doc.theme = BOKEH_THEME
+
+
 def individual_nprocessing_doc(scheduler, extra, doc):
     current_load = CurrentLoad(scheduler, sizing_mode="stretch_both")
     current_load.update()
@@ -1618,27 +1651,7 @@ class BokehScheduler(BokehServer):
 
         self.server_kwargs["prefix"] = prefix or None
 
-        self.apps = {
-            "/system": systemmonitor_doc,
-            "/stealing": stealing_doc,
-            "/workers": workers_doc,
-            "/events": events_doc,
-            "/counters": counters_doc,
-            "/tasks": tasks_doc,
-            "/status": status_doc,
-            "/profile": profile_doc,
-            "/profile-server": profile_server_doc,
-            "/graph": graph_doc,
-            "/individual-task-stream": individual_task_stream_doc,
-            "/individual-progress": individual_progress_doc,
-            "/individual-graph": individual_graph_doc,
-            "/individual-profile": individual_profile_doc,
-            "/individual-profile-server": individual_profile_server_doc,
-            "/individual-nbytes": individual_nbytes_doc,
-            "/individual-nprocessing": individual_nprocessing_doc,
-            "/individual-workers": individual_workers_doc,
-        }
-
+        self.apps = applications
         self.apps = {k: partial(v, scheduler, self.extra) for k, v in self.apps.items()}
 
         self.loop = io_loop or scheduler.loop
@@ -1667,3 +1680,33 @@ class BokehScheduler(BokehServer):
         ]
 
         self.server._tornado.add_handlers(r".*", handlers)
+
+
+applications = {
+    "/system": systemmonitor_doc,
+    "/stealing": stealing_doc,
+    "/workers": workers_doc,
+    "/events": events_doc,
+    "/counters": counters_doc,
+    "/tasks": tasks_doc,
+    "/status": status_doc,
+    "/profile": profile_doc,
+    "/profile-server": profile_server_doc,
+    "/graph": graph_doc,
+    "/individual-task-stream": individual_task_stream_doc,
+    "/individual-progress": individual_progress_doc,
+    "/individual-graph": individual_graph_doc,
+    "/individual-profile": individual_profile_doc,
+    "/individual-profile-server": individual_profile_server_doc,
+    "/individual-nbytes": individual_nbytes_doc,
+    "/individual-cpu": individual_cpu_doc,
+    "/individual-nprocessing": individual_nprocessing_doc,
+    "/individual-workers": individual_workers_doc,
+}
+
+try:
+    import pynvml  # noqa: 1708
+except ImportError:
+    pass
+else:
+    from . import nvml  # noqa: 1708
