@@ -4,33 +4,21 @@ import copy
 import weakref
 
 from tornado import gen
-from dask.utils import format_bytes
 
 from .cluster import Cluster
-from ..comm import connect
 from ..core import rpc, CommClosedError
-from ..utils import (
-    log_errors,
-    LoopRunner,
-    silence_logging,
-    ignoring,
-    Log,
-    Logs,
-    PeriodicCallback,
-    format_dashboard_link,
-)
+from ..utils import LoopRunner, silence_logging, ignoring
 from ..scheduler import Scheduler
 from ..security import Security
 
 
 class ProcessInterface:
-    """ An interface for Scheduler and Worker processes for use in SpecCluster
-
-    Parameters
-    ----------
-    loop:
-        A pointer to the running loop.
+    """
+    An interface for Scheduler and Worker processes for use in SpecCluster
 
+    This interface is responsible to submit a worker or scheduler process to a
+    resource manager like Kubernetes, Yarn, or SLURM/PBS/SGE/...
+    It should implement the methods below, like ``start`` and ``close``
     """
 
     def __init__(self):
@@ -49,16 +37,37 @@ class ProcessInterface:
         return _().__await__()
 
     async def start(self):
-        """ Start the process. """
+        """ Submit the process to the resource manager
+
+        For workers this doesn't have to wait until the process actually starts,
+        but can return once the resource manager has the request, and will work
+        to make the job exist in the future
+
+        For the scheduler we will expect the scheduler's ``.address`` attribute
+        to be avaialble after this completes.
+        """
         self.status = "running"
 
     async def close(self):
-        """ Close the process. """
+        """ Close the process
+
+        This will be called by the Cluster object when we scale down a node,
+        but only after we ask the Scheduler to close the worker gracefully.
+        This method should kill the process a bit more forcefully and does not
+        need to worry about shutting down gracefully
+        """
         self.status = "closed"
 
     def __repr__(self):
         return "<%s: status=%s>" % (type(self).__name__, self.status)
 
+    async def __aenter__(self):
+        await self
+        return self
+
+    async def __aexit__(self, *args, **kwargs):
+        await self.close()
+
 
 class SpecCluster(Cluster):
     """ Cluster that requires a full specification of workers
@@ -171,11 +180,8 @@ class SpecCluster(Cluster):
         self.new_spec = copy.copy(worker)
         self.workers = {}
         self._i = 0
-        self._asynchronous = asynchronous
         self.security = security or Security()
         self.scheduler_comm = None
-        self.scheduler_info = {}
-        self.periodic_callbacks = {}
 
         if silence_logs:
             self._old_logging_level = silence_logging(level=silence_logs)
@@ -183,16 +189,16 @@ class SpecCluster(Cluster):
         self._loop_runner = LoopRunner(loop=loop, asynchronous=asynchronous)
         self.loop = self._loop_runner.loop
 
-        self.status = "created"
         self._instances.add(self)
         self._correct_state_waiting = None
         self._name = name or type(self).__name__
 
+        super().__init__(asynchronous=asynchronous)
+
         if not self.asynchronous:
             self._loop_runner.start()
             self.sync(self._start)
             self.sync(self._correct_state)
-            self.sync(self._wait_for_workers)
 
     async def _start(self):
         while self.status == "starting":
@@ -222,37 +228,7 @@ class SpecCluster(Cluster):
             self.scheduler.address,
             connection_args=self.security.get_connection_args("client"),
         )
-        comm = await connect(
-            self.scheduler_address,
-            connection_args=self.security.get_connection_args("client"),
-        )
-        await comm.write({"op": "subscribe_worker_status"})
-        self.scheduler_info = await comm.read()
-        self._watch_worker_status_comm = comm
-        self._watch_worker_status_task = asyncio.ensure_future(
-            self._watch_worker_status(comm)
-        )
-        self.status = "running"
-
-    async def _watch_worker_status(self, comm):
-        """ Listen to scheduler for updates on adding and removing workers """
-        while True:
-            try:
-                msgs = await comm.read()
-            except OSError:
-                break
-
-            for op, msg in msgs:
-                if op == "add":
-                    workers = msg.pop("workers")
-                    self.scheduler_info["workers"].update(workers)
-                    self.scheduler_info.update(msg)
-                elif op == "remove":
-                    del self.scheduler_info["workers"][msg]
-                else:
-                    raise ValueError("Invalid op", op, msg)
-
-        await comm.close()
+        await super()._start()
 
     def _correct_state(self):
         if self._correct_state_waiting:
@@ -306,7 +282,6 @@ class SpecCluster(Cluster):
             await self._correct_state()
             if self.workers:
                 await asyncio.wait(list(self.workers.values()))  # maybe there are more
-            await self._wait_for_workers()
             return self
 
         return _().__await__()
@@ -323,13 +298,6 @@ class SpecCluster(Cluster):
                 raise gen.TimeoutError("Worker unexpectedly closed")
             await asyncio.sleep(0.1)
 
-    async def __aenter__(self):
-        await self
-        return self
-
-    async def __aexit__(self, typ, value, traceback):
-        await self.close()
-
     async def _close(self):
         while self.status == "closing":
             await asyncio.sleep(0.1)
@@ -337,37 +305,23 @@ class SpecCluster(Cluster):
             return
         self.status = "closing"
 
-        for pc in self.periodic_callbacks.values():
-            pc.stop()
-
         self.scale(0)
         await self._correct_state()
         async with self._lock:
             with ignoring(CommClosedError):
                 await self.scheduler_comm.close(close_workers=True)
+
         await self.scheduler.close()
-        await self._watch_worker_status_comm.close()
-        await self._watch_worker_status_task
         for w in self._created:
             assert w.status == "closed"
-        self.scheduler_comm.close_rpc()
 
         if hasattr(self, "_old_logging_level"):
             silence_logging(self._old_logging_level)
 
-        self.status = "closed"
-
-    def close(self, timeout=None):
-        with ignoring(RuntimeError):  # loop closed during process shutdown
-            return self.sync(self._close, callback_timeout=timeout)
-
-    def __del__(self):
-        if self.status != "closed":
-            self.loop.add_callback(self.close)
+        await super()._close()
 
     def __enter__(self):
         self.sync(self._correct_state)
-        self.sync(self._wait_for_workers)
         assert self.status == "running"
         return self
 
@@ -376,6 +330,13 @@ class SpecCluster(Cluster):
         self._loop_runner.stop()
 
     def scale(self, n):
+        if len(self.worker_spec) > n:
+            not_yet_launched = set(self.worker_spec) - {
+                v["name"] for v in self.scheduler_info["workers"].values()
+            }
+            while len(self.worker_spec) > n and not_yet_launched:
+                del self.worker_spec[not_yet_launched.pop()]
+
         while len(self.worker_spec) > n:
             self.worker_spec.popitem()
 
@@ -411,12 +372,9 @@ class SpecCluster(Cluster):
         return not not self.new_spec
 
     async def scale_down(self, workers):
-        workers = set(workers)
-
-        for k, v in self.workers.items():
-            if getattr(v, "worker_address", v.address) in workers:
-                del self.worker_spec[k]
-
+        for w in workers:
+            if w in self.worker_spec:
+                del self.worker_spec[w]
         await self
 
     scale_up = scale  # backwards compatibility
@@ -428,155 +386,6 @@ class SpecCluster(Cluster):
             len(self.workers),
         )
 
-    async def _logs(self, scheduler=True, workers=True):
-        logs = Logs()
-
-        if scheduler:
-            L = await self.scheduler_comm.logs()
-            logs["Scheduler"] = Log("\n".join(line for level, line in L))
-
-        if workers:
-            d = await self.scheduler_comm.worker_logs(workers=workers)
-            for k, v in d.items():
-                logs[k] = Log("\n".join(line for level, line in v))
-
-        return logs
-
-    def logs(self, scheduler=True, workers=True):
-        """ Return logs for the scheduler and workers
-
-        Parameters
-        ----------
-        scheduler : boolean
-            Whether or not to collect logs for the scheduler
-        workers : boolean or Iterable[str], optional
-            A list of worker addresses to select.
-            Defaults to all workers if `True` or no workers if `False`
-
-        Returns
-        -------
-        logs: Dict[str]
-            A dictionary of logs, with one item for the scheduler and one for
-            each worker
-        """
-        return self.sync(self._logs, scheduler=scheduler, workers=workers)
-
-    @property
-    def dashboard_link(self):
-        try:
-            port = self.scheduler_info["services"]["dashboard"]
-        except KeyError:
-            return ""
-        else:
-            host = self.scheduler_address.split("://")[1].split(":")[0]
-            return format_dashboard_link(host, port)
-
-    def _widget_status(self):
-        workers = len(self.scheduler_info["workers"])
-        cores = sum(v["nthreads"] for v in self.scheduler_info["workers"].values())
-        memory = sum(v["memory_limit"] for v in self.scheduler_info["workers"].values())
-        memory = format_bytes(memory)
-        text = """
-<div>
-  <style scoped>
-    .dataframe tbody tr th:only-of-type {
-        vertical-align: middle;
-    }
-
-    .dataframe tbody tr th {
-        vertical-align: top;
-    }
-
-    .dataframe thead th {
-        text-align: right;
-    }
-  </style>
-  <table style="text-align: right;">
-    <tr> <th>Workers</th> <td>%d</td></tr>
-    <tr> <th>Cores</th> <td>%d</td></tr>
-    <tr> <th>Memory</th> <td>%s</td></tr>
-  </table>
-</div>
-""" % (
-            workers,
-            cores,
-            memory,
-        )
-        return text
-
-    def _widget(self):
-        """ Create IPython widget for display within a notebook """
-        try:
-            return self._cached_widget
-        except AttributeError:
-            pass
-
-        from ipywidgets import Layout, VBox, HBox, IntText, Button, HTML, Accordion
-
-        layout = Layout(width="150px")
-
-        if self.dashboard_link:
-            link = '<p><b>Dashboard: </b><a href="%s" target="_blank">%s</a></p>\n' % (
-                self.dashboard_link,
-                self.dashboard_link,
-            )
-        else:
-            link = ""
-
-        title = "<h2>%s</h2>" % type(self).__name__
-        title = HTML(title)
-        dashboard = HTML(link)
-
-        status = HTML(self._widget_status(), layout=Layout(min_width="150px"))
-
-        if self._supports_scaling:
-            request = IntText(0, description="Workers", layout=layout)
-            scale = Button(description="Scale", layout=layout)
-
-            minimum = IntText(0, description="Minimum", layout=layout)
-            maximum = IntText(0, description="Maximum", layout=layout)
-            adapt = Button(description="Adapt", layout=layout)
-
-            accordion = Accordion(
-                [HBox([request, scale]), HBox([minimum, maximum, adapt])],
-                layout=Layout(min_width="500px"),
-            )
-            accordion.selected_index = None
-            accordion.set_title(0, "Manual Scaling")
-            accordion.set_title(1, "Adaptive Scaling")
-
-            def adapt_cb(b):
-                self.adapt(minimum=minimum.value, maximum=maximum.value)
-
-            adapt.on_click(adapt_cb)
-
-            def scale_cb(b):
-                with log_errors():
-                    n = request.value
-                    with ignoring(AttributeError):
-                        self._adaptive.stop()
-                    self.scale(n)
-
-            scale.on_click(scale_cb)
-        else:
-            accordion = HTML("")
-
-        box = VBox([title, HBox([status, accordion]), dashboard])
-
-        self._cached_widget = box
-
-        def update():
-            status.value = self._widget_status()
-
-        pc = PeriodicCallback(update, 500, io_loop=self.loop)
-        self.periodic_callbacks["cluster-repr"] = pc
-        pc.start()
-
-        return box
-
-    def _ipython_display_(self, **kwargs):
-        return self._widget()._ipython_display_(**kwargs)
-
 
 @atexit.register
 def close_clusters():
