@@ -9,7 +9,7 @@ def serialize_keras_model(model):
 
     if keras.__version__ < "1.2.0":
         raise ImportError(
-            "Need Keras >= 1.2.0. " "Try pip install keras --upgrade --no-deps"
+            "Need Keras >= 1.2.0. Try pip install keras --upgrade --no-deps"
         )
 
     header = model._updated_config()
