@@ -1,6 +1,7 @@
 from distributed import Pub, Sub
 from distributed.utils_test import gen_cluster
 
+import asyncio
 import toolz
 from tornado import gen
 import pytest
@@ -22,7 +23,7 @@ def test_basic(c, s, a, b):
         sub = Sub("a")
         return list(toolz.take(5, sub))
 
-    c.run(publish, workers=[a.address])
+    asyncio.ensure_future(c.run(publish, workers=[a.address]))
 
     tasks = [c.submit(f, i) for i in range(4)]
     results = yield c.gather(tasks)
