@@ -1,7 +1,7 @@
-from __future__ import print_function, division, absolute_import
-
+import asyncio
 import atexit
 from collections import deque
+from concurrent.futures import ThreadPoolExecutor
 from contextlib import contextmanager
 from datetime import timedelta
 import functools
@@ -11,13 +11,14 @@ import json
 import logging
 import multiprocessing
 from numbers import Number
-import operator
 import os
 import re
 import shutil
 import socket
 from time import sleep
-from importlib import import_module
+import importlib
+from importlib.util import cache_from_source
+import inspect
 import sys
 import tempfile
 import threading
@@ -26,8 +27,7 @@ import weakref
 import pkgutil
 import six
 import tblib.pickling_support
-
-from .compatibility import cache_from_source, getargspec, invalidate_caches, reload
+import xml.etree.ElementTree
 
 try:
     import resource
@@ -49,7 +49,7 @@ try:
 except ImportError:
     PollIOLoop = None  # dropped in tornado 6.0
 
-from .compatibility import PY3, PY2, get_thread_identity, unicode
+from .compatibility import PYPY, WINDOWS
 from .metrics import time
 
 
@@ -65,7 +65,9 @@ no_default = "__no_default__"
 
 
 def _initialize_mp_context():
-    if PY3 and not sys.platform.startswith("win") and "PyPy" not in sys.version:
+    if WINDOWS or PYPY:
+        return multiprocessing
+    else:
         method = dask.config.get("distributed.worker.multiprocessing-method")
         ctx = multiprocessing.get_context(method)
         # Makes the test suite much faster
@@ -73,10 +75,7 @@ def _initialize_mp_context():
         if "pkg_resources" in sys.modules:
             preload.append("pkg_resources")
         ctx.set_forkserver_preload(preload)
-    else:
-        ctx = multiprocessing
-
-    return ctx
+        return ctx
 
 
 mp_context = _initialize_mp_context()
@@ -98,7 +97,7 @@ def has_arg(func, argname):
     """
     while True:
         try:
-            if argname in getargspec(func).args:
+            if argname in inspect.getfullargspec(func).args:
                 return True
         except TypeError:
             break
@@ -200,8 +199,7 @@ def ignore_exceptions(coroutines, *exceptions):
     raise gen.Return(results)
 
 
-@gen.coroutine
-def All(args, quiet_exceptions=()):
+async def All(args, quiet_exceptions=()):
     """ Wait on many tasks at the same time
 
     Err once any of the tasks err.
@@ -214,11 +212,11 @@ def All(args, quiet_exceptions=()):
     quiet_exceptions: tuple, Exception
         Exception types to avoid logging if they fail
     """
-    tasks = gen.WaitIterator(*args)
+    tasks = gen.WaitIterator(*map(asyncio.ensure_future, args))
     results = [None for _ in args]
     while not tasks.done():
         try:
-            result = yield tasks.next()
+            result = await tasks.next()
         except Exception:
 
             @gen.coroutine
@@ -237,13 +235,11 @@ def All(args, quiet_exceptions=()):
 
             quiet()
             raise
-
         results[tasks.current_index] = result
-    raise gen.Return(results)
+    return results
 
 
-@gen.coroutine
-def Any(args, quiet_exceptions=()):
+async def Any(args, quiet_exceptions=()):
     """ Wait on many tasks at the same time and return when any is finished
 
     Err once any of the tasks err.
@@ -254,11 +250,11 @@ def Any(args, quiet_exceptions=()):
     quiet_exceptions: tuple, Exception
         Exception types to avoid logging if they fail
     """
-    tasks = gen.WaitIterator(*args)
+    tasks = gen.WaitIterator(*map(asyncio.ensure_future, args))
     results = [None for _ in args]
     while not tasks.done():
         try:
-            result = yield tasks.next()
+            result = await tasks.next()
         except Exception:
 
             @gen.coroutine
@@ -280,7 +276,7 @@ def Any(args, quiet_exceptions=()):
 
         results[tasks.current_index] = result
         break
-    raise gen.Return(results)
+    return results
 
 
 def sync(loop, func, *args, callback_timeout=None, **kwargs):
@@ -300,14 +296,14 @@ def sync(loop, func, *args, callback_timeout=None, **kwargs):
         pass
 
     e = threading.Event()
-    main_tid = get_thread_identity()
+    main_tid = threading.get_ident()
     result = [None]
     error = [False]
 
     @gen.coroutine
     def f():
         try:
-            if main_tid == get_thread_identity():
+            if main_tid == threading.get_ident():
                 raise RuntimeError("sync() called from thread of running loop")
             yield gen.moment
             thread_state.asynchronous = True
@@ -554,6 +550,7 @@ def is_kernel():
 hex_pattern = re.compile("[a-f]+")
 
 
+@functools.lru_cache(100000)
 def key_split(s):
     """
     >>> key_split('x')
@@ -608,102 +605,48 @@ def key_split(s):
         return "Other"
 
 
-try:
-    from functools import lru_cache
-except ImportError:
-    lru_cache = False
-    pass
-else:
-    key_split = lru_cache(100000)(key_split)
+def key_split_group(x):
+    """A more fine-grained version of key_split
 
-if PY3:
-
-    def key_split_group(x):
-        """A more fine-grained version of key_split
-
-        >>> key_split_group('x')
-        'x'
-        >>> key_split_group('x-1')
-        'x-1'
-        >>> key_split_group('x-1-2-3')
-        'x-1-2-3'
-        >>> key_split_group(('x-2', 1))
-        'x-2'
-        >>> key_split_group("('x-2', 1)")
-        'x-2'
-        >>> key_split_group('hello-world-1')
-        'hello-world-1'
-        >>> key_split_group(b'hello-world-1')
-        'hello-world-1'
-        >>> key_split_group('ae05086432ca935f6eba409a8ecd4896')
-        'data'
-        >>> key_split_group('<module.submodule.myclass object at 0xdaf372')
-        'myclass'
-        >>> key_split_group(None)
-        'Other'
-        >>> key_split_group('x-abcdefab')  # ignores hex
-        'x-abcdefab'
-        """
-        typ = type(x)
-        if typ is tuple:
-            return x[0]
-        elif typ is str:
-            if x[0] == "(":
-                return x.split(",", 1)[0].strip("()\"'")
-            elif len(x) == 32 and re.match(r"[a-f0-9]{32}", x):
-                return "data"
-            elif x[0] == "<":
-                return x.strip("<>").split()[0].split(".")[-1]
-            else:
-                return x
-        elif typ is bytes:
-            return key_split_group(x.decode())
+    >>> key_split_group('x')
+    'x'
+    >>> key_split_group('x-1')
+    'x-1'
+    >>> key_split_group('x-1-2-3')
+    'x-1-2-3'
+    >>> key_split_group(('x-2', 1))
+    'x-2'
+    >>> key_split_group("('x-2', 1)")
+    'x-2'
+    >>> key_split_group('hello-world-1')
+    'hello-world-1'
+    >>> key_split_group(b'hello-world-1')
+    'hello-world-1'
+    >>> key_split_group('ae05086432ca935f6eba409a8ecd4896')
+    'data'
+    >>> key_split_group('<module.submodule.myclass object at 0xdaf372')
+    'myclass'
+    >>> key_split_group(None)
+    'Other'
+    >>> key_split_group('x-abcdefab')  # ignores hex
+    'x-abcdefab'
+    """
+    typ = type(x)
+    if typ is tuple:
+        return x[0]
+    elif typ is str:
+        if x[0] == "(":
+            return x.split(",", 1)[0].strip("()\"'")
+        elif len(x) == 32 and re.match(r"[a-f0-9]{32}", x):
+            return "data"
+        elif x[0] == "<":
+            return x.strip("<>").split()[0].split(".")[-1]
         else:
-            return "Other"
-
-
-else:
-
-    def key_split_group(x):
-        """A more fine-grained version of key_split
-
-        >>> key_split_group('x')
-        'x'
-        >>> key_split_group('x-1')
-        'x-1'
-        >>> key_split_group('x-1-2-3')
-        'x-1-2-3'
-        >>> key_split_group(('x-2', 1))
-        'x-2'
-        >>> key_split_group("('x-2', 1)")
-        'x-2'
-        >>> key_split_group('hello-world-1')
-        'hello-world-1'
-        >>> key_split_group(b'hello-world-1')
-        'hello-world-1'
-        >>> key_split_group('ae05086432ca935f6eba409a8ecd4896')
-        'data'
-        >>> key_split_group('<module.submodule.myclass object at 0xdaf372')
-        'myclass'
-        >>> key_split_group(None)
-        'Other'
-        >>> key_split_group('x-abcdefab')  # ignores hex
-        'x-abcdefab'
-        """
-        typ = type(x)
-        if typ is tuple:
-            return x[0]
-        elif typ is str or typ is unicode:
-            if x[0] == "(":
-                return x.split(",", 1)[0].strip("()\"'")
-            elif len(x) == 32 and re.match(r"[a-f0-9]{32}", x):
-                return "data"
-            elif x[0] == "<":
-                return x.strip("<>").split()[0].split(".")[-1]
-            else:
-                return x
-        else:
-            return "Other"
+            return x
+    elif typ is bytes:
+        return key_split_group(x.decode())
+    else:
+        return "Other"
 
 
 @contextmanager
@@ -805,14 +748,14 @@ def tokey(o):
     --------
 
     >>> tokey(b'x')
-    'x'
+    b'x'
     >>> tokey('x')
     'x'
     >>> tokey(1)
     '1'
     """
     typ = type(o)
-    if typ is unicode or typ is bytes:
+    if typ is str or typ is bytes:
         return o
     else:
         return str(o)
@@ -822,7 +765,7 @@ def validate_key(k):
     """Validate a key as received on a stream.
     """
     typ = type(k)
-    if typ is not unicode and typ is not bytes:
+    if typ is not str and typ is not bytes:
         raise TypeError("Unexpected key type %s (value: %r)" % (typ, k))
 
 
@@ -972,7 +915,7 @@ def ensure_bytes(s):
         return s
     if isinstance(s, memoryview):
         return s.tobytes()
-    if isinstance(s, bytearray) or PY2 and isinstance(s, buffer):  # noqa: F821
+    if isinstance(s, bytearray):  # noqa: F821
         return bytes(s)
     if hasattr(s, "encode"):
         return s.encode()
@@ -1077,13 +1020,13 @@ def import_file(path):
     if not names_to_import:
         logger.warning("Found nothing to import from %s", filename)
     else:
-        invalidate_caches()
+        importlib.invalidate_caches()
         if tmp_python_path is not None:
             sys.path.insert(0, tmp_python_path)
         try:
             for name in names_to_import:
                 logger.info("Reload module %s from %s file", name, ext)
-                loaded.append(reload(import_module(name)))
+                loaded.append(importlib.reload(importlib.import_module(name)))
         finally:
             if tmp_python_path is not None:
                 sys.path.remove(tmp_python_path)
@@ -1212,7 +1155,7 @@ def parse_timedelta(s, default="seconds"):
     >>> parse_timedelta('300ms')
     0.3
     >>> parse_timedelta(timedelta(seconds=3))  # also supports timedeltas
-    3
+    3.0
     """
     if s is None:
         return None
@@ -1263,32 +1206,15 @@ def asciitable(columns, rows):
     return "\n".join([bar, header, bar, data, bar])
 
 
-if PY2:
-
-    def nbytes(frame, _bytes_like=(bytes, bytearray, buffer)):  # noqa: F821
-        """ Number of bytes of a frame or memoryview """
-        if isinstance(frame, _bytes_like):
-            return len(frame)
-        elif isinstance(frame, memoryview):
-            if frame.shape is None:
-                return frame.itemsize
-            else:
-                return functools.reduce(operator.mul, frame.shape, frame.itemsize)
-        else:
+def nbytes(frame, _bytes_like=(bytes, bytearray)):
+    """ Number of bytes of a frame or memoryview """
+    if isinstance(frame, _bytes_like):
+        return len(frame)
+    else:
+        try:
             return frame.nbytes
-
-
-else:
-
-    def nbytes(frame, _bytes_like=(bytes, bytearray)):
-        """ Number of bytes of a frame or memoryview """
-        if isinstance(frame, _bytes_like):
+        except AttributeError:
             return len(frame)
-        else:
-            try:
-                return frame.nbytes
-            except AttributeError:
-                return len(frame)
 
 
 def PeriodicCallback(callback, callback_time, io_loop=None):
@@ -1397,7 +1323,6 @@ if "asyncio" in sys.modules and tornado.version_info[0] >= 5:
         )
 
     if not jupyter_event_loop_initialized:
-        import asyncio
         import tornado.platform.asyncio
 
         asyncio.set_event_loop_policy(
@@ -1405,18 +1330,9 @@ if "asyncio" in sys.modules and tornado.version_info[0] >= 5:
         )
 
 
+@functools.lru_cache(1000)
 def has_keyword(func, keyword):
-    if PY3:
-        return keyword in inspect.signature(func).parameters
-    else:
-        # https://stackoverflow.com/questions/50100498/determine-keywords-of-a-tornado-coroutine
-        if gen.is_coroutine_function(func):
-            func = func.__wrapped__
-        return keyword in inspect.getargspec(func).args
-
-
-if lru_cache:
-    has_keyword = lru_cache(1000)(has_keyword)
+    return keyword in inspect.signature(func).parameters
 
 
 # from bokeh.palettes import viridis
@@ -1489,3 +1405,85 @@ def format_dashboard_link(host, port):
     else:
         scheme = "http"
     return template.format(scheme=scheme, host=host, port=port, **os.environ)
+
+
+def is_coroutine_function(f):
+    return asyncio.iscoroutinefunction(f) or gen.is_coroutine_function(f)
+
+
+class Log(str):
+    """ A container for logs """
+
+    def _repr_html_(self):
+        return "<pre><code>\n{log}\n</code></pre>".format(log=self.rstrip())
+
+
+class Logs(dict):
+    """ A container for multiple logs """
+
+    def _repr_html_(self):
+        summaries = [
+            "<details>\n<summary>{title}</summary>\n{log}\n</details>".format(
+                title=title, log=log._repr_html_()
+            )
+            for title, log in self.items()
+        ]
+        return "\n\n".join(summaries)
+
+
+def cli_keywords(d: dict, cls=None):
+    """ Convert a kwargs dictionary into a list of CLI keywords
+
+    Parameters
+    ----------
+    d: dict
+        The keywords to convert
+    cls: callable
+        The callable that consumes these terms to check them for validity
+
+    Examples
+    --------
+    >>> cli_keywords({"x": 123, "save_file": "foo.txt"})
+    ['--x', '123', '--save-file', 'foo.txt']
+
+    >>> from dask.distributed import Worker
+    >>> cli_keywords({"x": 123}, Worker)
+    Traceback (most recent call last):
+    ...
+    ValueError: Class distributed.worker.Worker does not support keyword x
+    """
+    if cls:
+        for k in d:
+            if not has_keyword(cls, k):
+                raise ValueError(
+                    "Class %s does not support keyword %s" % (typename(cls), k)
+                )
+
+    def convert_value(v):
+        out = str(v)
+        if " " in out and "'" not in out and '"' not in out:
+            out = '"' + out + '"'
+        return out
+
+    return sum(
+        [["--" + k.replace("_", "-"), convert_value(v)] for k, v in d.items()], []
+    )
+
+
+def is_valid_xml(text):
+    return xml.etree.ElementTree.fromstring(text) is not None
+
+
+try:
+    _offload_executor = ThreadPoolExecutor(
+        max_workers=1, thread_name_prefix="Dask-Offload"
+    )
+except TypeError:
+    _offload_executor = ThreadPoolExecutor(max_workers=1)
+
+weakref.finalize(_offload_executor, _offload_executor.shutdown)
+
+
+@gen.coroutine
+def offload(fn, *args, **kwargs):
+    return (yield _offload_executor.submit(fn, *args, **kwargs))
